#!/usr/bin/ruby
# $Id: epubgen.rb 253 2010-12-28 19:24:17Z briane $

require "./datatypes.rb"

class SectionData
    attr_accessor :year, :month, :filename, :title, :uniqueid
    @year = 0
    @month = 0
    @filename = ""
    @title = ""
    @uniqueid = ""
end

class EpubGen

def initialize
    system("rm -rf ./output") if File.exists?("output")
    Dir.mkdir("output")
    Dir.mkdir("output/OPS")
    Dir.mkdir("output/META-INF")
    @sections = Hash.new
    @section_keys = Array.new
    @book_unique_id = 'WikiLeaksCableGateAutomaticEpub'
    @book_title = 'WikiLeaks Cables'
    @book_author = 'WikiLeaks'
    @book_isbn = '666666666X'
    @zip_location = `which zip`
    @zip_location.chomp!
    raise "Unable to find the 'zip' command line application. Do you have it installed?" if @zip_location.length == 0
end

def AddSection(wiki_article_data)
    sd = SectionData.new
    sd.year = wiki_article_data.year
    sd.month = wiki_article_data.month
    sd.title = wiki_article_data.title
    sd.filename = File.basename(wiki_article_data.original_filename)
    sd.uniqueid = File.basename(wiki_article_data.original_filename).sub('\..*', '') # without extension
    key = "#{sd.year}#{sd.month}#{sd.title.upcase}"
    f = File.new("./output/OPS/" + File.basename(wiki_article_data.original_filename), "w")
    f << '<?xml version="1.0" encoding="UTF-8" ?>' + "\n"
    f << '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">' + "\n"
    f << '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">' << "\n"
    f << '<head>' + "\n"
    f << '<meta http-equiv="Content-Type" content="application/xhtml+xml; charset=utf-8" />' + "\n"
    f << "<title>" << wiki_article_data.title << "</title></head><body>"
    f << wiki_article_data.html_content
    f << "</body></html>"
    f.close
    #print "#{sd.inspect}\n\n"
    @sections[key] = sd
    #print "#{@sections.inspect}\n\n"
end

def WriteMetadata
    # sort by key (i.e. year+month+title)
    @section_keys = @sections.keys.sort
    WriteTOC()
    WriteSpine()
    WriteNCX()
    WriteManifest()
end

def Compress
    system("cp 1000px-Wikileaks_logo.jpg ./output/iTunesArtwork")
    File.unlink("wikileaks.epub") if File.exists?("wikileaks.epub")
    command = "cd output ; zip -0 ../wikileaks.epub mimetype ; cd .."
    rc = system(command)
    raise "Failed during first zip step" if rc != true
    command = "cd output ; zip -u -9 -r ../wikileaks.epub META-INF OPS iTunesArtwork ; cd .."
    rc = system(command)
    raise "Failed during second zip step" if rc != true
    print "\n\n\n\n\n"
    print "Your epub file is complete and located at " + Dir.getwd + "/wikileaks.epub\n"
    print "\n\n"
end

def WriteTOC
    previous_year = 0
    f = File.new("./output/OPS/index.html", "w")
    f << '<?xml version="1.0" encoding="UTF-8" ?>' + "\n"
    f << '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">' + "\n"
    f << '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">' << "\n"
    f << '<head>' + "\n"
    f << '<meta http-equiv="Content-Type" content="application/xhtml+xml; charset=utf-8" />' + "\n"
    f << "<title>" << @book_title << "</title></head><body>\n"
    f << "<h1>WikiLeaks Cable Index</h1>\n"
    f << "<ul>\n"
    @section_keys.each { |key|
        value = @sections[key]
        if 0 == previous_year
            previous_year = value.year
            f << "\n\n    <li>" << value.year << "</li><ul>\n"
        end
        if previous_year != value.year
            previous_year = value.year
            f << "    </ul>\n\n    <li>" << value.year << "</li><ul>\n"
        end
        f << "        <li>"
        f << '<a href="' << value.filename << '">'
        f << value.year << "-" << value.month << ": " << value.title
        f << '</a>'
        f << "</li>\n"
    }
    f << "        </ul>\n    </ul>\n"
    f << "</body></html>"
    f.close
end
private :WriteTOC

def WriteSpine
    f = File.new("./output/OPS/" + File.basename("index.opf"), "w")
    f << '<?xml version="1.0"?>' + "\n"
    f << '<package version="2.0" xmlns="http://www.idpf.org/2007/opf" unique-identifier="' << @book_unique_id << '">' + "\n"
    f << '<metadata xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf">' + "\n"
    f << '  <dc:title>' << @book_title << '</dc:title>' + "\n"
    f << '  <dc:language>en</dc:language>' + "\n"
    f << '  <dc:identifier id="BookId" opf:scheme="ISBN">' << @book_isbn << '</dc:identifier>' + "\n"
    f << '  <dc:creator opf:file-as="' << @book_author << '" opf:role="aut">' << @book_author << '</dc:creator>' + "\n"
    f << '</metadata>' + "\n"
    f << '<manifest>' + "\n"
    f << '  <item id="index" href="index.html" media-type="application/xhtml+xml"/>' + "\n"
    @section_keys.each { |key|
        value = @sections[key]
        f << '  <item id="' << value.uniqueid << '" href="' << value.filename << '" media-type="application/xhtml+xml"/>' + "\n"
    }
    f << '  <item id="ncx" href="index.ncx" media-type="application/x-dtbncx+xml"/>' + "\n"
    f << '</manifest>' + "\n"
    f << '<spine toc="ncx">' + "\n"
    f << '  <itemref idref="index" />' + "\n"
    @section_keys.each { |key|
        value = @sections[key]
        f << '  <itemref idref="' << value.uniqueid << '" />' + "\n"
    }
    f << '</spine>' + "\n"
    f << '<guide>' + "\n"
    f << '  <reference type="toc" title="Table of Contents" href="index.html" />' + "\n"
    f << '</guide>' + "\n"
    f << '</package>' + "\n"
    f.close
end
private :WriteSpine

def WriteNCX
    f = File.new("./output/OPS/" + File.basename("index.ncx"), "w")
    f << '<?xml version="1.0" encoding="UTF-8"?>' + "\n"
    f << '<!DOCTYPE ncx PUBLIC "-//NISO//DTD ncx 2005-1//EN" "http://www.daisy.org/z3986/2005/ncx-2005-1.dtd">' + "\n"
    f << '<ncx version="2005-1" xml:lang="en" xmlns="http://www.daisy.org/z3986/2005/ncx/">' + "\n"
    f << '<head>' + "\n"
    f << '  <meta name="dtb:uid" content="' << @book_isbn << '"/> <!-- same as in .opf -->' + "\n"
    f << '  <meta name="dtb:depth" content="1"/> <!-- 1 or higher -->' + "\n"
    f << '  <meta name="dtb:totalPageCount" content="0"/> <!-- must be 0 -->' + "\n"
    f << '  <meta name="dtb:maxPageNumber" content="0"/> <!-- must be 0 -->' + "\n"
    f << '</head>' + "\n"
    f << '<docTitle>' + "\n"
    f << '  <text>' << @book_title << '</text>' + "\n"
    f << '</docTitle>' + "\n"
    f << '<docAuthor>' + "\n"
    f << '  <text>' << @book_author << '</text>' + "\n"
    f << '</docAuthor>' + "\n"
    f << '<navMap>' + "\n"
    counter = 1
    @section_keys.each { |key|
        value = @sections[key]
        f << '  <item id="' << value.uniqueid << '" href="' << value.filename << '" media-type="application/xhtml+xml"/>' + "\n"
        f << '  <navPoint class="chapter" id="' << value.uniqueid << '" playOrder="' << counter << '">' + "\n"
        f << '    <navLabel><text>' << value.year << "-" << value.month << ": " << value.title << '</text></navLabel>' + "\n"
        f << '    <content src="' << value.filename << '"/>' + "\n"
        f << '  </navPoint>' + "\n"
        counter += 1
    }
    f << '</navMap>' + "\n"
    f << '</ncx>' + "\n"
    f.close
end
private :WriteNCX

def WriteManifest
    f = File.new("./output/mimetype", "w")
    f << "application/epub+zip"
    f.close
    f = File.new("./output/META-INF/container.xml", "w")
    f << '<?xml version="1.0" encoding="UTF-8" ?>' + "\n"
    f << '<container version="1.0" xmlns="urn:oasis:names:tc:opendocument:xmlns:container">' + "\n"
    f << '<rootfiles>' + "\n"
    f << '  <rootfile full-path="OPS/index.opf" media-type="application/oebps-package+xml"/>' + "\n"
    f << '</rootfiles>' + "\n"
    f << '</container>' + "\n"
    f.close
end
private :WriteManifest


end #EpubGen
